//= require <dSS/util/Util>
//= require <dSS/devices/VisibilityMatrix>
//= require <dSS/devices/OutputModeForm>
//= require <dSS/devices/ScenePropertyForm>
//= require <dSS/devices/GroupPushbuttonForm>
//= require <dSS/devices/JokerConfigForm>
//= require <dSS/devices/ShadePropertyForm>
//= require <dSS/devices/OperationTypeForm>
//= require <dSS/devices/MultiButtonConfigForm>

// parameters:
//      maskElement: element to mask when opening the window
//      iconCls: icon class for the title bar
//      extraTitle: string to append after the default title
//      standardForms []: array of forms for the standard view
//      advancedForms []: array of forms for the advanced view
//      name: device name
//      dsid: device dsid
//      loaders []: array ofloader objects that will load
//                  data which has to be passed to all available forms
//
//                  forms that do not depend on the global information
//                  may load their internal data right away, others
//                  can wait for the "dataloaded" event that will be
//                  sent once all globalLoader objects have retreived
//                  their data


dSS.devprops.DevicePropertiesWindow = Ext.extend(Ext.Window, {
    constructor: function(config) {
        if (config.dsid === undefined) {
           throw dSS._("GenericDevicePropertiesWindow: missing dSID");
        }

        var items = [];
        var i;

        this.closed = false;

        if (config.showDefaults === true) {
            var info = dSS.util.CreateDefaultDeviceInfoPanel(config.name,
                                                         config.dsid);
            items.push(info);
        }

        if (config.standardForms === undefined) {
            config.standardForms = [];
        }

        this.expectedChildEvents = config.standardForms.length;
        this.receivedSaveEvents = 0;
        this.receivedFormLoadEvents = 0;
        this.showSave = false;

        if (config.standardForms.length > 1) {
            for (i = 0; i < config.standardForms.length; i++) {
                if (i === 0) {
                    config.standardForms[i].baseCls =
                                                    'x-panel-no-bottom-border';
                } else if (i === config.standardForms.length - 1) {
                    config.standardForms[i].baseCls = 'x-panel-no-top-border';
                } else {
                    config.standardForms[i].baseCls =
                            'x-panel-no-top-border x-panel-no-bottom-border';
                }

                config.standardForms[i].baseCls += ' x-panel';
            }
        }

        if (config.standardForms.length > 0) {
            items = items.concat(config.standardForms);
        }

        this.loaders = [];
        this.loaderIndex = 0;
        this.expectedLoadEvents = 0;
        this.receivedLoadEvents = 0;

        if (config.loaders !== undefined) {
            this.loaders = config.loaders;
            this.expectedLoadEvents = config.loaders.length;
        }

        if (config.advancedForms !== undefined) {

            if (config.advancedForms.length > 1) {
                for (i = 0; i < config.advancedForms.length; i++) {
                    if (i === 0) {
                        config.advancedForms[i].baseCls =
                                                    'x-panel-no-bottom-border';
                    } else if (i === config.advancedForms.length - 1) {
                        config.advancedForms[i].baseCls =
                                                    'x-panel-no-top-border';
                    } else {
                        config.advancedForms[i].baseCls =
                            'x-panel-no-top-border x-panel-no-bottom-border';
                    }

                    config.advancedForms[i].baseCls += ' x-panel';
                }
            }


            if (config.advancedForms.length > 0) {

                items.push({
                                xtype: 'box',
                                height: 10
                           });
                items.push({
                                xtype: 'panel',
                                title: dSS._("Show advanced properties"),
                                collapsed: true,
                                ref: 'advanced',
                                collapsible: true,
                                collapseFirst: true,
                                animCollapse: false,
                                floatable: false,
                                titleCollapse: true,
                                listeners:
                                {
                                    collapse: function(p) {
                                        p.setTitle(
                                          dSS._("Show advanced properties"));
                                    },
                                    expand: function(p) {
                                        p.setTitle(
                                          dSS._("Hide advanced properties"));
                                    }
                                },
                                items: [ config.advancedForms ]
                        });
                this.expectedChildEvents = this.expectedChildEvents +
                                           config.advancedForms.length;
            }
        }

        this.addEvents('formsubmitted', 'formloaded', 'loadcomplete',
                       'forcereload');

        this.saveButton = new Ext.Button({
            text: dSS._("Save"),
            disabled: true
        });

        this.cancelButton = {
            xtype: 'button',
            id: 'cancelbutton',
            ref: 'cancelbutton',
            text: dSS._("Cancel"),
            scope: this,
            handler: function() {
                if ((this.loaderIndex < this.loaders.length) &&
                    (this.loaderIndex >= 0) &&
                    (this.loaders[this.loaderIndex-1].tid !== undefined)) {
                    Ext.Ajax.abort(this.loaders[this.loaderIndex-1].tid);
                }
                this.loaders = [];
                this.maskElement.unmask();
                this.closed = true;
                this.close();
            }
        };

        var title = dSS._("Device properties");

        if (config.extraTitle !== undefined) {
            title = title + ' ' + config.extraTitle;
        }

        Ext.apply(this, {
            layout: 'table',
            layoutConfig: {
                columns: 1,
                tableAttrs: {
                    width: '100%'
                }
            },
            padding: 10,
            width: 400,
            resizable: false,
            iconCls: config.icon,
            title: title,
            items: items,
            buttons: [ this.saveButton, this.cancelButton ]
        });

        dSS.devprops.DevicePropertiesWindow.superclass.constructor.apply(this,
                arguments);

        this.maskElement.mask();

        this.saveButton.on('click', function() {
                this.el.mask();
                this.receivedSaveEvents = 0;
                if (this.advanced !== undefined) {
                    this.advanced.items.each(function(item) {
                        item.fireEvent('submit');
                    });
                }
                this.items.each(function(item) { item.fireEvent('submit'); });
            },
            this);

        this.on('afterrender',
            function() {
                if (this.loaders.length > 0) {
                    this.body.mask(dSS._("Loading properties") + '...',
                                   'x-mask-loading');
                    this.loadNext();
                }
            },
            this);

        this.on('beforedestroy',
            function(component) {
                this.maskElement.unmask();
            },
            this);

        this.on('formloaded',
            function(submitOK) {
                if (submitOK === true) {
                    this.showSave = true;
                }
                this.receivedFormLoadEvents++;
                if (this.receivedFormLoadEvents >= this.expectedChildEvents) {
                    this.receivedFormLoadEvents = 0;
                    if (this.showSave === true) {
                        this.saveButton.enable();
                    }
                }
            },
            this);

        this.on('formsubmitted',
            function() {
                this.receivedSaveEvents++;
                if (this.receivedSaveEvents >= this.expectedChildEvents) {
                    this.maskElement.unmask();
                    this.closed = true;
                    this.close();
                }
            },
            this);

        this.on('forcereload', this.reloadData.createDelegate(this), this);

        this.on('close', function() {
                this.closed = true;
            },
            this);

    },
    reloadData: function() {
        if (this.closed === true) {
            return;
        }

        if (this.loading === true) {
            if (this.loaders.length > 0) {
                this.reloadData.defer(500, this);
            }
            return;
        }

        if (this.loaders.length > 0) {
            this.body.mask(dSS._("Loading properties") + '...',
                           'x-mask-loading');
            this.receivedFormLoadEvents = 0;
            this.receivedLoadEvents = 0;
            this.loaderIndex = 0;
            this.showSave = false;
            this.saveButton.disable();
            this.loadNext();
        }
    },
    loadNext: function() {
        if (this.closed === true) {
            return;
        }
        this.loading = true;
        if (this.loaderIndex < this.loaders.length) {
            this.loaders[this.loaderIndex].addListener('loadcomplete',
                                        this.loadComplete.createDelegate(this),
                                        this, { single: true });

            this.loaders[this.loaderIndex].load();
            this.loaderIndex++;
        }
    },
    saveSettings: function(param1, param2) {
        this.el.mask();
        this.receivedSaveEvents = 0;
        this.items.each(function(item) { item.fireEvent('submit'); });
        if (this.advanced !== undefined) {
            this.advanced.items.each(function(item) {
                item.fireEvent('dataloaded', param1, param2);
            });
        }
    },
    loadComplete: function(data) {
        this.receivedLoadEvents++;
        this.items.each(function(item) {
            item.fireEvent('dataloaded', data);
        });

        if (this.advanced !== undefined) {
            this.advanced.items.each(function(item) {
                item.fireEvent('dataloaded', data);
            });
        }

        if (this.receivedLoadEvents >= this.expectedLoadEvents) {
            this.body.unmask();
            this.loading = false;
        } else {
            this.loadNext();
        }
    }
});

Ext.reg('dssdevscenepropswindow', dSS.devprops.DevicePropertiesWindow);

